<?php

namespace Teknisa\CTB\Services;

use Teknisa\CTB\Factories\Ctb14000EncerramentoExercicioFactory;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Model\Contabilidade\Entities\Lista;
use Teknisa\Libs\Util\Environment;

class Ctb14000EncerramentoExercicioService {

    protected $entityManager;
    protected $environment;
    protected $nrorg;
    protected $factoryEncerramento;
    protected $newCode;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Environment $environment, Ctb14000EncerramentoExercicioFactory $factoryEncerramento, NewCode $newCode) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->nrorg = $environment->getNrorgTrab();
        $this->factoryEncerramento = $factoryEncerramento;
        $this->newCode = $newCode;
    }
    
    public function formataData($data){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
    
    public function betweenDates($compareDate, $beginDate, $endDate) {
        // É necessário truncar a data, removendo a hora, para que isso não
        // influencie na comparação.
        $beginDate = DateUtil::truncateData($beginDate);
        $endDate = DateUtil::truncateData($endDate);
        // Depois de truncada, a data final deve ser incrementada em um dia,
        // para que o último dia parametrizado também possa ser considerado.
        $endDate = DateUtil::adicionaIntervalo($endDate, 1, DateUtil::DIA);

        // Retorna true se a data não for menor que a primeira ou maior que a segunda.
        return !(DateUtil::comparaDatas($compareDate, $beginDate, DateUtil::ANTES) || DateUtil::comparaDatas($compareDate, $endDate, DateUtil::DEPOIS));
    }

    public function verificaCdLote($row) {
        $existeCapaLote = $this->factoryEncerramento->verificaCdLote(array(
            "CDLOTE" => $row['CDLOTE'],
            "CDEMPRESA" => $row['CDEMPRESA'],
            "NRORG" => $this->nrorg
        ));
        if ($existeCapaLote !== false) {
            throw new \Exception("Lote informado já existe.");
        }
    }

    public function geraLote($row){
        $this->factoryEncerramento->geraLote($row);
    }

    public function getMask(){
        $nrorg = array('NRORG' => $this->nrorg);
        $result = $this->factoryEncerramento->getMask($nrorg);
        return $result['CDPICTCCTB'];
    }
    
    public function geraNovoCodigo(
        $cdcontador, $nrorg, $sequencial, $tamanho
    ){
        try {
            /** @var Novocodigo $novocodigo */
            $novocodigo = $this->newCode->getCodeByTableName($cdcontador, $nrorg, $sequencial, $tamanho);
            return $novocodigo;
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }
    
    public function saveContResul($contas, $encerramento){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            // Busca Dados Cabeçalho
            $dtencerramento = $encerramento['DTENCERRAMENTO'];
            $idtiposaldo    = $encerramento['IDTIPOSALDO'];
            $cdempresa      = $encerramento['CDEMPRESA'];
            $cdcontdest     = $encerramento['CDCONTCTBL'];
            
            $cdcontctbl = $contas['CDCONTCTBL'];
            
            //$dtencerramento = $this->formataData($encerramento['DTENCERRAMENTO']);
            $nrorg = $this->environment->getNrOrgTrab();
            
            //$newNrlanclote = $this->geraNovoCodigo('LANCLOTE'.$cdempresa.$cdlote, $nrorg, 1, 6);
            
            // verificar data primeiro
            $params = array(
                "P_CDEMPRESA" => $encerramento["CDEMPRESA"],
                "NRORG" => $nrorg
            );
            
            $datas = $this->factoryEncerramento->buscaDatasContResul($params);
            
            $dataExisteCont = $this->betweenDates($this->formataData($dtencerramento), $this->formataData($datas["DTINICCONT"]), $this->formataData($datas["DTFINACONT"]));
            $dataExisteLanc = $this->betweenDates($this->formataData($dtencerramento), $this->formataData($datas["DTINICLANC"]), $this->formataData($datas["DTFINALANC"]));
            
            if(!$dataExisteCont){
                throw new \Exception("Data de transferência fora do período contábil.");
            }
            
            if(!$dataExisteLanc){
                throw new \Exception("Data de Transferência fora do período de lançamento.");
            }
        
            // verificar se tem contas com saldo
            $params = array(
                "P_DTDATA" => $encerramento["DTENCERRAMENTO"],
                "P_IDSALDO" => $encerramento["IDTIPOSALDO"],
                "P_CDEMPRESA" => $encerramento["CDEMPRESA"],
                "P_CDCONTDEST" => $encerramento["CDCONTCTBL"],
                "P_CDCONTCTBL" => $contas["CDCONTCTBL"],
                "NRORG" => $nrorg
            );

            $contasResultado = $this->factoryEncerramento->buscaContResul($params);
            // testar saldo
            if(!$contasResultado){
                throw new \Exception("Não existe saldo para a data informada.");
            }
            //var_dump($contasResultado);//die();
            
            return $contasResultado;
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function deleteContResul($row) {
        $params = array(
            "NRLISTA" => $row["NRLISTA"],
            "NRORG" => $this->environment->getNrOrgTrab()
        );

        $this->lancamentosFactory->deleteLista($params);
    }
    
    public function finalizaContResul($contas, $encerramento){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $params = array(
                "CDEMPRESA" => $encerramento['CDEMPRESA'],
                "CDLOTE" => $encerramento['CDLOTE'],
                "NRORG" => $this->environment->getNrOrgTrab()
            );
            $existeCdlote = $this->factoryEncerramento->verificaCdLote($params);
            if($existeCdlote){
                throw new \Exception("Código do lote existe.");
            }else{
                $nrorg = $this->environment->getNrOrgTrab();
                $cdempresa   = $encerramento['CDEMPRESA'];
                $cdcontadest = $encerramento['CDCONTCTBL'];
                $cdhistpadr  = $encerramento['CDHISTPADR'];
                $cdlote      = $encerramento['CDLOTE'];
    
                $dtencerramento = $encerramento['DTENCERRAMENTO'];
                $idencerramen    = $encerramento['IDTIPOSALDO'];
                
                $dtsaldo = $dataFinalMes = DateUtil::getUltimoDiaDoMes($this->formataData($dtencerramento));
                $dtsaldo = DateUtil::truncateString($dtsaldo);
                
                $newLista = $this->geraNovoCodigo('LISTA', $nrorg, 1, 10);
                $nrlista = $newLista;
                //var_dump($nrlista);
                foreach($contas as $conta){
                    $cdlista = $conta["CDCONTCTBL"];
                    
                    $lista = $this->factoryEncerramento->insereLista($nrorg, $nrlista, $cdlista);
                    $this->entityManager->persist($lista);
                    $this->entityManager->flush();
                    //var_dump($lista);
                    
                }
                
                $this->factoryEncerramento->execProcTRANSFERE_SALDO_CONTCTBL($cdempresa, $cdcontadest, $cdhistpadr, $dtsaldo, $cdlote, $nrlista, $idencerramen);
                //var_dump("CDEMPRESA: ".$cdempresa." CDCONTADEST: ".$cdcontadest." CDHISTPADR: ".$cdhistpadr." DTSALDO: ".$dtsaldo." CDLOTE: ".$cdlote." NRLISTA: ".$nrlista." IDENCERRAMEN: ".$idencerramen);
                
                $params = array(
                    "NRLISTA" => $nrlista,
                    "NRORG" => $nrorg
                );
            
                $this->factoryEncerramento->deleteLista($params);
                
                $this->entityManager->flush();
                $connection->commit();
                
                //return $contasResultado;
               
            }
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        
    }
}